@@ -61,9 +61,9 @@ KdePlatformTheme::KdePlatformTheme()
 {
     loadSettings();
 
-    if (KWindowSystem::isPlatformWayland()) {
+    // explicitly not KWindowSystem::isPlatformWayland to not include the kwin process
+    if (QGuiApplication::platformName() == QLatin1String("wayland")) {
         m_kwaylandIntegration.reset(new KWaylandIntegration());
-        m_kwaylandIntegration->init();
     }
 
 #if HAVE_X11
@@ -79,7 +79,9 @@ KdePlatformTheme::KdePlatformTheme()
     // contextual tooltips instead; we only want to ever see the titlebar button
     // to invoke the "What's This?" feature in 3rd-party Qt apps that have set
     // "What's This" help text and requested the button
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QCoreApplication::setAttribute(Qt::AA_DisableWindowContextHelpButton, true);
+#endif
 
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar, false);
     setQtQuickControlsTheme();
